* FROM: https://stats.idre.ucla.edu/sas/dae/ordinal-logistic-regression/;

* Generic session options - page and line size, no page breaks;
OPTIONS nonumber nodate nocenter pagesize=MAX linesize=120 formdlim=' ' spool; 									
* Kill default titles on output, ODS tables in log, turn on graphics; 
TITLE; ODS TRACE OFF; ODS LISTING; ODS GRAPHICS ON;

********************************************************************************
*******             BEGIN DATA MANIPULATION FOR EXAMPLE 1                *******
*******             CHANGE "filesave" to your directory                  *******
********************************************************************************

* Location for original files for these models  change this path;
%LET filesave= C:\Dropbox\20_PSQF7375_Generalized\PSQF7375_Generalized_Example1; 
LIBNAME filesave "&filesave.";

/* Send data to STATA;
PROC EXPORT DATA=filesave.ologit OUTFILE="&filesave.\ologit.dta" DBMS=STATA REPLACE; RUN; */

* Creating formats for categorical variables;
PROC FORMAT; 
    VALUE F2apply   0="0No" 1="1Pry";
    VALUE F3apply   0="0No" 1="1Eh" 2="2Very";
    VALUE FparentGD 0="0NoDegree" 1="1YesDegree";
    VALUE Fprivate  0="0public" 1="1private";
RUN;
* Import data into work library, transform variables, apply value formats;
DATA work.ologit; SET filesave.ologit;
	IF apply=0 THEN apply2=0;
	IF apply>0 THEN apply2=1;
     parentGD=pared;
     GPA3=GPA-3;
          IF public=1 THEN private=0; 
     ELSE IF public=0 THEN private=1;
     ELSE IF public=. THEN private=.;
     LABEL apply=    "apply: 0=Not, 1=Eh, 2=Very"
		 apply2=   "apply2: 0=No, 1=Pry"
           parentGD= "parentGD: Parent Has Graduate Degree (0=N,1=Y)"
           private=  "private: Student Attends Private University (0=N,1=Y)"
           GPA3=     "GPA3: Student GPA (0=3)";  
	FORMAT apply2 F2apply. apply F3apply. parentGD FparentGD. private Fprivate.;
RUN;

********************************************************************************
******                 BEGIN EXAMPLE 1 PART 1 MODELS				   *******
********************************************************************************

* Open output directory to save results to;
ODS RTF FILE="&filesave.\Example1_Part1_SAS_Output.rtf" STYLE=HTMLBlue STARTPAGE=NO;

TITLE1 "DESCRIPTIVES FOR STUDY VARIABLES";
PROC MEANS NDEC=2 DATA=work.ologit; 
     VAR GPA; 
RUN;
PROC FREQ DATA=work.ologit;
     TABLE parentGD*private apply2 apply / NOROW NOCOL; 
RUN; TITLE1;

TITLE1 "SAS EMPTY MODEL PREDICTING BINARY DV";
PROC GLIMMIX DATA=work.ologit NOCLPRINT GRADIENT METHOD=MSPL; 
* DESCENDING means predict prob=1 rather than prob=0;
MODEL apply2 (DESCENDING) = / SOLUTION LINK=LOGIT DIST=BINARY;
* ILINK requests logit estimate to be transformed into probability;
ESTIMATE "Intercept" intercept 1 / ILINK;
RUN; TITLE1; 

TITLE1 "SAS MAIN EFFECTS MODEL PREDICTING BINARY DV";
PROC GLIMMIX DATA=work.ologit NOCLPRINT GRADIENT METHOD=MSPL;  
CLASS parentGD private;
MODEL apply2 (DESCENDING) = GPA3 parentGD private 
                            / SOLUTION LINK=LOGIT DIST=BINARY ODDSRATIO(AT GPA3=0 LABEL);
CONTRAST "Wald test of R2" GPA3 1, parentGD -1 1, private -1 1 / CHISQ;
ESTIMATE "Intercept for No Degree,  Public,  GPA=2" intercept 1 GPA3 -1 parentGD 1 0 private 1 0 / ILINK;
ESTIMATE "Intercept for No Degree,  Private, GPA=2" intercept 1 GPA3 -1 parentGD 1 0 private 0 1 / ILINK;
ESTIMATE "Intercept for Yes Degree, Public,  GPA=2" intercept 1 GPA3 -1 parentGD 0 1 private 1 0 / ILINK;
ESTIMATE "Intercept for Yes Degree, Private, GPA=2" intercept 1 GPA3 -1 parentGD 0 1 private 0 1 / ILINK;
ESTIMATE "Intercept for No Degree,  Public,  GPA=3" intercept 1 GPA3  0 parentGD 1 0 private 1 0 / ILINK;
ESTIMATE "Intercept for No Degree,  Private, GPA=3" intercept 1 GPA3  0 parentGD 1 0 private 0 1 / ILINK;
ESTIMATE "Intercept for Yes Degree, Public,  GPA=3" intercept 1 GPA3  0 parentGD 0 1 private 1 0 / ILINK;
ESTIMATE "Intercept for Yes Degree, Private, GPA=3" intercept 1 GPA3  0 parentGD 0 1 private 0 1 / ILINK;
ESTIMATE "Intercept for No Degree,  Public,  GPA=4" intercept 1 GPA3  1 parentGD 1 0 private 1 0 / ILINK;
ESTIMATE "Intercept for No Degree,  Private, GPA=4" intercept 1 GPA3  1 parentGD 1 0 private 0 1 / ILINK;
ESTIMATE "Intercept for Yes Degree, Public,  GPA=4" intercept 1 GPA3  1 parentGD 0 1 private 1 0 / ILINK;
ESTIMATE "Intercept for Yes Degree, Private, GPA=4" intercept 1 GPA3  1 parentGD 0 1 private 0 1 / ILINK;
ESTIMATE "Slope for GPA" GPA3 1 / ILINK; * Example of non-sense ILINK for a slope; 
RUN; TITLE1; 

TITLE1 "SAS INTERACTIONS MODEL PREDICTING BINARY DV";
PROC GLIMMIX DATA=work.ologit NOCLPRINT GRADIENT METHOD=MSPL;  
CLASS parentGD private;
MODEL apply2 (DESCENDING) = GPA3|parentGD|private@2 / SOLUTION LINK=LOGIT DIST=BINARY 
                                                      ODDSRATIO(AT GPA3=0 DIFF=ALL LABEL);
CONTRAST "Wald test of R2" GPA3 1, parentGD -1 1, private -1 1, 
                           GPA3*parentGD -1 1, GPA3*private -1 1, 
                           parentGD*private -1 1 1 -1 / CHISQ;
CONTRAST "Wald test of change in R2" GPA3*parentGD -1 1, GPA3*private -1 1,
                                     parentGD*private -1 1 1 -1 / CHISQ;
* Conditional means for plotting interactions, SLICEDIFF gives simple effects;
LSMEANS parentGD*private / ILINK SLICEDIFF=parentGD SLICEDIFF=private ODDSRATIO CL AT(GPA3)=(-1); * At GPA=2;
LSMEANS parentGD*private / ILINK SLICEDIFF=parentGD SLICEDIFF=private ODDSRATIO CL AT(GPA3)=( 0); * At GPA=3;
LSMEANS parentGD*private / ILINK SLICEDIFF=parentGD SLICEDIFF=private ODDSRATIO CL AT(GPA3)=( 1); * At GPA=4;
* Simple slopes for GPA and slope differences by moderators (EXP to get odds ratios);
ESTIMATE "GPA Slope for No  Degree for Public School"  GPA3 1 GPA3*parentGD  1 0 GPA3*private  1 0 / CL EXP;
ESTIMATE "GPA Slope for Yes Degree for Public School"  GPA3 1 GPA3*parentGD  0 1 GPA3*private  1 0 / CL EXP;
ESTIMATE "GPA Slope for No  Degree for Private School" GPA3 1 GPA3*parentGD  1 0 GPA3*private  0 1 / CL EXP;
ESTIMATE "GPA Slope for Yes Degree for Private School" GPA3 1 GPA3*parentGD  0 1 GPA3*private  0 1 / CL EXP;
ODS OUTPUT LSMEANS=work.Meansave SliceDiffs=work.Diffsave; * Save to combine output tables;
RUN; TITLE1; 

* Close output;
ODS RTF CLOSE; 


********************************************************************************
******                 BEGIN EXAMPLE 1 PART 2 MODELS				   *******
********************************************************************************

* Open output directory to save results to;
ODS RTF FILE="&filesave.\Example1_Part2_SAS_Output.rtf" STYLE=HTMLBlue STARTPAGE=NO;

TITLE1 "SAS EMPTY MODEL PREDICTING ORDINAL DV";
PROC GLIMMIX DATA=work.ologit NOCLPRINT GRADIENT METHOD=MSPL; 
MODEL apply (DESCENDING) = / SOLUTION LINK=CLOGIT DIST=MULT;
ESTIMATE "Intercept for y>1 (=2)"   intercept 1  0 / ILINK; * Highest is first;
ESTIMATE "Intercept for y>0 (=1+2)" intercept 0  1 / ILINK;
RUN; TITLE1; 

TITLE1 "SAS MAIN EFFECTS MODEL PREDICTING ORDINAL DV";
PROC GLIMMIX DATA=work.ologit NOCLPRINT GRADIENT METHOD=QUAD; 
MODEL apply (DESCENDING) = GPA3 parentGD private / SOLUTION LINK=CLOGIT DIST=MULT 
                                                   ODDSRATIO(AT GPA3=0 LABEL);
CONTRAST "Wald test of R2" GPA3 1, parentGD 1, private 1 / CHISQ;
ESTIMATE "y > 1 Int for No  Degree, Public,  GPA=2" int 1 0 GPA3 -1 parentGD 0 private 0 / ILINK;
ESTIMATE "y > 1 Int for No  Degree, Private, GPA=2" int 1 0 GPA3 -1 parentGD 0 private 1 / ILINK;
ESTIMATE "y > 1 Int for Yes Degree, Public,  GPA=2" int 1 0 GPA3 -1 parentGD 1 private 0 / ILINK;
ESTIMATE "y > 1 Int for Yes Degree, Private, GPA=2" int 1 0 GPA3 -1 parentGD 1 private 1 / ILINK;
ESTIMATE "y > 1 Int for No  Degree, Public,  GPA=3" int 1 0 GPA3  0 parentGD 0 private 0 / ILINK;
ESTIMATE "y > 1 Int for No  Degree, Private, GPA=3" int 1 0 GPA3  0 parentGD 0 private 1 / ILINK;
ESTIMATE "y > 1 Int for Yes Degree, Public,  GPA=3" int 1 0 GPA3  0 parentGD 1 private 0 / ILINK;
ESTIMATE "y > 1 Int for Yes Degree, Private, GPA=3" int 1 0 GPA3  0 parentGD 1 private 1 / ILINK;
ESTIMATE "y > 1 Int for No  Degree, Public,  GPA=4" int 1 0 GPA3  1 parentGD 0 private 0 / ILINK;
ESTIMATE "y > 1 Int for No  Degree, Private, GPA=4" int 1 0 GPA3  1 parentGD 0 private 1 / ILINK;
ESTIMATE "y > 1 Int for Yes Degree, Public,  GPA=4" int 1 0 GPA3  1 parentGD 1 private 0 / ILINK;
ESTIMATE "y > 1 Int for Yes Degree, Private, GPA=4" int 1 0 GPA3  1 parentGD 1 private 1 / ILINK;

ESTIMATE "y > 0 Int for No  Degree, Public,  GPA=2" int 0 1 GPA3 -1 parentGD 0 private 0 / ILINK;
ESTIMATE "y > 0 Int for No  Degree, Private, GPA=2" int 0 1 GPA3 -1 parentGD 0 private 1 / ILINK;
ESTIMATE "y > 0 Int for Yes Degree, Public,  GPA=2" int 0 1 GPA3 -1 parentGD 1 private 0 / ILINK;
ESTIMATE "y > 0 Int for Yes Degree, Private, GPA=2" int 0 1 GPA3 -1 parentGD 1 private 1 / ILINK;
ESTIMATE "y > 0 Int for No  Degree, Public,  GPA=3" int 0 1 GPA3  0 parentGD 0 private 0 / ILINK;
ESTIMATE "y > 0 Int for No  Degree, Private, GPA=3" int 0 1 GPA3  0 parentGD 0 private 1 / ILINK;
ESTIMATE "y > 0 Int for Yes Degree, Public,  GPA=3" int 0 1 GPA3  0 parentGD 1 private 0 / ILINK;
ESTIMATE "y > 0 Int for Yes Degree, Private, GPA=3" int 0 1 GPA3  0 parentGD 1 private 1 / ILINK;
ESTIMATE "y > 0 Int for No  Degree, Public,  GPA=4" int 0 1 GPA3  1 parentGD 0 private 0 / ILINK;
ESTIMATE "y > 0 Int for No  Degree, Private, GPA=4" int 0 1 GPA3  1 parentGD 0 private 1 / ILINK;
ESTIMATE "y > 0 Int for Yes Degree, Public,  GPA=4" int 0 1 GPA3  1 parentGD 1 private 0 / ILINK;
ESTIMATE "y > 0 Int for Yes Degree, Private, GPA=4" int 0 1 GPA3  1 parentGD 1 private 1 / ILINK;
ESTIMATE "Slope for GPA" GPA3 1 / ILINK; * Example of non-sense ILINK for a slope; 
RUN; TITLE1; 

TITLE1 "SAS MAIN EFFECTS MODEL TESTING ORDINAL PROPORTIONAL ODDS";
TITLE2 "MUST USE LOGISTIC INSTEAD OF GLIMMIX";
PROC LOGISTIC DATA=work.ologit; 
MODEL apply (DESCENDING) = GPA3 parentGD private / LINK=CLOGIT UNEQUALSLOPES;
* Request multiv Wald test of model R2;
Model:     TEST GPA3_2Very=0, GPA3_1Eh=0, parentGD_2Very=0,
                parentGD_1Eh=0, private_2Very=0, private_1Eh=0;
* Requesting tests of differences between slopes and DF=3 test;
GPA3:      TEST GPA3_2Very=GPA3_1Eh;
parentGD:  TEST parentGD_2Very=parentGD_1Eh;
private:   TEST private_2Very=private_1Eh;
PropOdds:  TEST GPA3_2Very=GPA3_1Eh, parentGD_2Very=parentGD_1Eh, private_2Very=private_1Eh;
RUN;

TITLE1 "SAS MAIN EFFECTS ORDINAL MODEL -- AUTOMATED SELECTION OF PROPORTIONAL ODDS";
PROC LOGISTIC DATA=work.ologit; 
MODEL apply (DESCENDING) = GPA3 parentGD private / LINK=CLOGIT 
/* Start with unequal slopes and remove if p>.05, keep 3 slopes minimum */
         EQUALSLOPES UNEQUALSLOPES SELECTION=BACKWARD STOP=3 DETAILS;
RUN;

TITLE1 "SAS MAIN EFFECTS MODEL -- CUSTOM SELECTION OF PROPORTIONAL ODDS";
PROC LOGISTIC DATA=work.ologit; 
MODEL apply (DESCENDING) = GPA3 parentGD private / LINK=CLOGIT UNEQUALSLOPES=private;
private: TEST private_2Very=private_1Eh;
RUN;

* MY TESTS TO MAKE SURE I UNDERSTOOD WHAT EACH PROGRAM DOES;
TITLE1 "SAS MAIN EFFECTS ORDINAL MODEL TESTING PROPORTIONAL ODDS";
TITLE2 "USING NLMIXED TO WRITE CUSTOM MODEL";
PROC NLMIXED DATA=work.ologit METHOD=GAUSS TECH=QUANEW GCONV=1e-12;
* Must list all parms to be estimated here with start values;
* B00 and B01 = intercepts for each equation;
* Bs = fixed effects, now separate per equation;
PARMS B01=-2.5 B00=-0.4 B11GPA3=0 B10GPA3=0 
      B21parentGD=0 B20parentGD=0 B31private=0 B30private=0;
* Linear predictor written as single-level equation for y>0 and y>1;
Y0 = B00 + B10GPA3*GPA3 + B20parentGD*parentGD + B30private*private;
Y1 = B01 + B11GPA3*GPA3 + B21parentGD*parentGD + B31private*private;
* Model for probability of response - writing it the shorter way;
     IF (apply=0) THEN P =  1 - (1/(1 + EXP(-Y0)));
ELSE IF (apply=1) THEN P =      (1/(1 + EXP(-Y0))) - (1/(1 + EXP(-Y1)));
ELSE IF (apply=2) THEN P =                           (1/(1 + EXP(-Y1)));
LL = LOG(P);
MODEL apply ~ GENERAL(LL);
* Testing proportional odds;
ESTIMATE "GPA Slope Diff"       B11GPA3     - B10GPA3;
ESTIMATE "ParentGD Slope Diff"  B21parentGD - B20parentGD;
ESTIMATE "Private Slope Diff"   B31private  - B30private;
CONTRAST "Overall Proportional Odds Test" B11GPA3-B10GPA3, 
          B21parentGD-B20parentGD, B31private-B30private;
RUN;


******************************** BEGIN NOMINAL MODELS ************************************;

TITLE1 "SAS EMPTY MODEL PREDICTING NOMINAL DV";
PROC GLIMMIX DATA=work.ologit NOCLPRINT GRADIENT METHOD=MSPL; 
MODEL apply (REFERENCE="1Eh") = / SOLUTION LINK=GLOGIT DIST=MULT;
* ILINK requests logit estimate to be transformed into probability;
ESTIMATE "Intercept" intercept 1 / ILINK BYCAT; * BYCAT gives per submodel;
RUN; TITLE1; 

TITLE1 "SAS MAIN EFFECTS MODEL PREDICTING NOMINAL DV";
PROC GLIMMIX DATA=work.ologit NOCLPRINT GRADIENT METHOD=MSPL; 
MODEL apply (REFERENCE="1Eh") = GPA3 parentGD private / SOLUTION LINK=GLOGIT DIST=MULT
										      ODDSRATIO(AT GPA3=0 LABEL);
CONTRAST "Model R2 multivariate Wald test" GPA3 1, parentGD 1, private 1 / CHISQ;
ESTIMATE "Int: No  Degree, Public,  GPA=2" int 1 GPA3 -1 parentGD 0 private 0 / ILINK BYCAT;
ESTIMATE "Int: No  Degree, Private, GPA=2" int 1 GPA3 -1 parentGD 0 private 1 / ILINK BYCAT;
ESTIMATE "Int: Yes Degree, Public,  GPA=2" int 1 GPA3 -1 parentGD 1 private 0 / ILINK BYCAT;
ESTIMATE "Int: Yes Degree, Private, GPA=2" int 1 GPA3 -1 parentGD 1 private 1 / ILINK BYCAT;
ESTIMATE "Int: No  Degree, Public,  GPA=3" int 1 GPA3  0 parentGD 0 private 0 / ILINK BYCAT;
ESTIMATE "Int: No  Degree, Private, GPA=3" int 1 GPA3  0 parentGD 0 private 1 / ILINK BYCAT;
ESTIMATE "Int: Yes Degree, Public,  GPA=3" int 1 GPA3  0 parentGD 1 private 0 / ILINK BYCAT;
ESTIMATE "Int: Yes Degree, Private, GPA=3" int 1 GPA3  0 parentGD 1 private 1 / ILINK BYCAT;
ESTIMATE "Int: No  Degree, Public,  GPA=4" int 1 GPA3  1 parentGD 0 private 0 / ILINK BYCAT;
ESTIMATE "Int: No  Degree, Private, GPA=4" int 1 GPA3  1 parentGD 0 private 1 / ILINK BYCAT;
ESTIMATE "Int: Yes Degree, Public,  GPA=4" int 1 GPA3  1 parentGD 1 private 0 / ILINK BYCAT;
ESTIMATE "Int: Yes Degree, Private, GPA=4" int 1 GPA3  1 parentGD 1 private 1 / ILINK BYCAT;
RUN; TITLE1; 

* Close output;
ODS RTF CLOSE; 

