* Prevent "more" messages from appearing
set more off
* Control line length
set linesize 150

********************************************************************************
*******             BEGIN DATA MANIPULATION FOR EXAMPLE 1                *******
*******             CHANGE "filesave" to your directory                  *******
********************************************************************************

* Defining global variable for file location to be replaced in code below
global filesave "C:\Dropbox\20_PSQF7375_Generalized\PSQF7375_Generalized_Example1"
 
* Import data, transform variables, apply value formats
use "$filesave\ologit.dta", clear
gen apply2=0
recode apply2 (0=1) if apply==1
recode apply2 (0=1) if apply==2
gen parentgd=pared
gen gpa3=gpa-3
gen private=0
recode private (0=1) if public==0
label variable apply    "apply: 0=Not, 1=Eh, 2=Very"
label variable apply2   "apply2: 0=No, 1=Pry"
label variable parentgd "parentgd: Parent Has Graduate Degree (0=N,1=Y)"
label variable private  "private: Student Attends Private University (0=N,1=Y)"
label variable gpa3     "gpa3: Student GPA (0=3)"
label define f2apply     0 "0No" 1 "1Pry"
label define f3apply     0 "0No" 1 "1Eh" 2 "2Very"
label define fparentgd   0 "0NoDegree" 1 "1YesDegree"
label define fprivate    0 "0public" 1 "1private"
label values apply2 f2apply
label values apply  f3apply
label values parentgd fparentgd
label values private fprivate
 
********************************************************************************
******                    BEGIN EXAMPLE 1 PART 1 MODELS					 *******
********************************************************************************

* Save results to separate file
log using $filesave\Example1_Part1_STATA_Output.log, replace name(Example1_Part1)
 
display as result "DESCRIPTIVES FOR STUDY VARIABLES"
summarize gpa
tabulate parentgd private
tabulate apply2 // For Part 1
tabulate apply  // For Part 2

display as result "STATA EMPTY MODEL PREDICTING BINARY DV"
melogit apply2 ,  
estat ic, n(400),
nlcom 1/(1+exp(-1*(_b[_cons]))) // intercept in probability

display as result "STATA MAIN EFFECTS MODEL PREDICTING BINARY DV"
melogit apply2 c.gpa3 ib(last).parentgd ib(last).private,
estat ic, n(400),
test (c.gpa3=0)(i0.parentgd=i1.parentgd)(i0.private=i1.private)     // demo of Wald test of R2
margins i.parentgd, over(i.private) at(c.gpa3=(-1(1)1)) predict(xb) // logits
margins i.parentgd, over(i.private) at(c.gpa3=(-1(1)1))             // probabilities	

* Must re-run with 'or' added to first line to get odds ratios
display as result "STATA MAIN EFFECTS MODEL PREDICTING BINARY DV"
display as result "STATA ODDS RATIOS INSTEAD OF LOGITS"
melogit apply2 c.gpa3 ib(last).parentgd ib(last).private, or

	
display as result "INTERACTIONS EFFECTS MODEL PREDICTING BINARY DV"
melogit apply2 c.gpa3 ib(last).parentgd ib(last).private ///
		       c.gpa3#ib(last).parentgd c.gpa3#ib(last).private ///
		       ib(last).parentgd#ib(last).private,
estat ic, n(400),
test (c.gpa3=0)(i0.parentgd=i1.parentgd)(i0.private=i1.private) ///
     (c.gpa3#i0.parentgd=c.gpa3#i1.parentgd) ///
     (c.gpa3#i0.private=c.gpa3#i1.private)   /// 
     (i0.parentgd#i0.private=i1.parentgd#i1.private) // demo of Wald test of R2
test (c.gpa3#i0.parentgd=c.gpa3#i1.parentgd) ///
     (c.gpa3#i0.private=c.gpa3#i1.private)   /// 
     (i0.parentgd#i0.private=i1.parentgd#i1.private) // Wald test for change in R2
margins i.parentgd#i.private, at(c.gpa3=(-1(1)1)) predict(xb) // predicted logits
margins i.parentgd#i.private, at(c.gpa3=(-1(1)1))             // predicted probabilities
margins i.private@i.parentgd, at(c.gpa3=(-1(1)1)) predict(xb) // simple effects in logits	
margins i.parentgd@i.private, at(c.gpa3=(-1(1)1)) predict(xb) // simple effects in logits
lincom c.gpa3*1 + c.gpa3#i0.parentgd*1 + c.gpa3#i0.private    // GPA slope for no degree, public
lincom c.gpa3*1 + c.gpa3#i1.parentgd*1 + c.gpa3#i0.private    // GPA slope for yes degree, public
lincom c.gpa3*1 + c.gpa3#i0.parentgd*1 + c.gpa3#i1.private    // GPA slope for no degree, private
lincom c.gpa3*1 + c.gpa3#i1.parentgd*1 + c.gpa3#i1.private    // GPA slope for yes degree, private
  
* Must re-run with 'or' added to first line to get odds ratios
display as result "INTERACTIONS EFFECTS MODEL PREDICTING BINARY DV"
display as result "STATA ODDS RATIOS INSTEAD OF LOGITS"
melogit apply2 c.gpa3 ib(last).parentgd ib(last).private ///
               c.gpa3#ib(last).parentgd c.gpa3#ib(last).private ///
			   ib(last).parentgd#ib(last).private, or
// Could not figure out how to get odds ratios for simple effects without retyping them in nlcom
	
* Close log	
log close Example1_Part1


********************************************************************************
******                    BEGIN EXAMPLE 1 PART 2 MODELS					 *******
********************************************************************************

* Save results to separate file
log using $filesave\Example1_Part2_STATA_Output.log, replace name(Example1_Part2)
 
 * Need to find and install a new package -- first run this:
*search gologit2
* Now click on first link under "web resources" in new window to install, 
* Then can use gologit2 below


display as result "STATA EMPTY MODEL PREDICTING ORDINAL DV"
display as result "(ME)OLOGIT USES THRESHOLDS (GIVE LOGIT OF LOWER CATEGORY), NOT INTERCEPTS"  
meologit apply ,  
estat ic, n(400), 
nlcom 1/(1+exp(-1*(_b[/cut2]))) // threshold for y<2 in probability
nlcom 1/(1+exp(-1*(_b[/cut1]))) // threshold for y<1 in probability

display as result "STATA EMPTY MODEL PREDICTING ORDINAL DV"
display as result "GOLOGIT2 USES INTERCEPTS (GIVE LOGIT OF HIGHER CATEGORY), NOT THRESHOLDS"  
gologit2 apply ,  
estat ic, n(400), 
margins


display as result "STATA MAIN EFFECTS PROPOTIONAL ODDS MODEL PREDICTING ORDINAL DV"
gologit2 apply c.gpa3 c.parentgd c.private, pl
estat ic, n(400),
test (c.gpa3=0)(c.parentgd=0)(c.private=0)  // demo of Wald test of R2
margins, at(c.gpa3=(-1(1)1) c.parentgd=(0(1)1) c.private=(0(1)1)) predict(xb) // logits of y>0
margins, at(c.gpa3=(-1(1)1) c.parentgd=(0(1)1) c.private=(0(1)1))         // all probabilities	
	
display as result "STATA MAIN EFFECTS PROPOTIONAL ODDS MODEL PREDICTING ORDINAL DV"
display as result "ODDS RATIOS INSTEAD OF LOGITS"
gologit2 apply c.gpa3 c.parentgd c.private, pl or


display as result "STATA MAIN EFFECTS MODEL TESTING ORDINAL PROPORTIONAL ODDS"
display as result "DIRECTLY PROVIDES EACH SLOPE AND DIFFERENCES IN SLOPES ACROSS SUBMODELS"
gologit2 apply c.gpa3 c.parentgd c.private, gamma  // test: Multiv Wald Model R2 test
test ([0No]gpa3=0)([1Eh]gpa3=0)([0No]parentgd=0)([1Eh]parentgd=0)([0No]private=0)([1Eh]private=0) 
test ([0No]gpa3=[1Eh]gpa3)([0No]parentgd=[1Eh]parentgd)([0No]private=[1Eh]private) // Global PO test
estat ic, n(400), 


display as result "STATA MAIN EFFECTS ORDINAL MODEL -- AUTOMATED SELECTION OF PROPORTIONAL ODDS"
gologit2 apply c.gpa3 c.parentgd c.private, gamma auto
estat ic, n(400),


display as result "STATA MAIN EFFECTS ORDINAL MODEL -- CUSTOM SELECTION OF PROPORTIONAL ODDS"
gologit2 apply c.gpa3 c.parentgd c.private, npl(c.private) gamma
estat ic, n(400), 

display as result "STATA MAIN EFFECTS ORDINAL MODEL -- CUSTOM SELECTION OF PROPORTIONAL ODDS"
display as result "ODDS RATIOS INSTEAD OF LOGITS"
gologit2 apply c.gpa3 c.parentgd c.private, npl(c.private) gamma or


******************************** BEGIN NOMINAL MODELS ************************************
	
display as result "STATA EMPTY MODEL PREDICTING NOMINAL DV"
mlogit apply , baseoutcome(1), 
estat ic, n(400), 
nlcom 1/(1+exp(-1*(_b[0:_cons]))) // intercept for y=0 instead of 1 in probability
nlcom 1/(1+exp(-1*(_b[2:_cons]))) // intercept for y=2 instead of 1 in probability
	
display as result "STATA MAIN EFFECTS MODEL PREDICTING NOMINAL DV"
mlogit apply c.gpa3 c.parentgd c.private, baseoutcome(1), 
estat ic, n(400), 
test ([0No]c.gpa3=0)([2Very]c.gpa3=0)([0No]c.parentgd=0)([2Very]c.parentgd=0) ///
     ([0No]c.private=0)([2Very]c.private=0) // test: Model R2 multivariate Wald test
margins, at(c.gpa3=(-1(1)1) c.parentgd=(0(1)1) c.private=(0(1)1)) predict(xb) // logits for 1 vs 0
margins, at(c.gpa3=(-1(1)1) c.parentgd=(0(1)1) c.private=(0(1)1))             // all probabilities	
	
display as result "STATA MAIN EFFECTS MODEL PREDICTING NOMINAL DV"
display as result "ODDS (RISK?) RATIOS INSTEAD OF LOGITS"
mlogit apply c.gpa3 c.parentgd c.private, baseoutcome(1) rrr
estat ic, n(400),	
	
* Close log	
log close Example1_Part2
